import bisect
from typing import Sequence, Tuple


class MultiFilePruner(object):
    def __init__(self, gap=0):
        self.files = {}
        self.gap = gap
    
    def add_file(self, filename):
        self.files[filename] = Pruner(filename, self.gap)
    
    def prune_all(self):
        for filename, pruner in self.files.items():
            pruner.prune()

class Pruner(object):
    
    def __init__(self, source_filename, gap=0):
        """
        Initialize a Pruner object with configuration given as arguments.
        
        :param gap: milliseconds to insert between trimmed pieces when joining them back.
        """
        self.sourcefile = source_filename
        self.gap = gap
        self._empty_spans()
    
    def add_spans(self, *spans: Sequence[Tuple[int, int]]) -> None:
        for span in spans:
            for existing_span in self.spans():
                print(existing_span, span)
                if self.is_overlapping(span, existing_span):
                    raise ValueError(f"Found an overlapping span: trying to add \"{list(span)}\", "
                                     f"but it overlaps with \"{list([existing_span])}\".")
            bisect.insort(self.span_starts, span[0])
            bisect.insort(self.span_ends, span[1])
    
    def _empty_spans(self):
        self.span_starts = []
        self.span_ends = []
            
    def spans(self):
        return zip(self.span_starts, self.span_ends)
    
    @classmethod
    def is_overlapping(cls, span1, spane2):
        s1, e1 = span1
        s2, e2 = spane2
        return s1 <= e2 and s2 <= e1
        

if __name__ == '__main__':
    p = Pruner(1)
    p.add_spans( (1,2), (5,6) , [1,3])



from setuptools import setup

with open('README.md') as readme:
    long_desc = readme.read()

with open('requirements.txt') as requirements:
    requires = requirements.readlines()

setup(
    name='concatrim',
    version='0.0.1.dev2',
    python_requires='>=3',
    packages=['concatrim'],
    url='https://github.com/keighrim/concatrim',
    license='MIT',
    author='Keigh Rim',
    author_email='krim@brandeis.edu',
    description='Python program to trim-then-concatenate A/V media files using ffmpeg',
    long_description=long_desc,
    long_description_content_type="text/markdown",
    install_requires=requires,
)



import unittest
import concatrim

class test_mediapruner(unittest.TestCase):
    
    def setUp(self):
        self.pruner = concatrim.Pruner(1000)
        
    def test_is_overlapping(self):
        self.assertFalse(self.pruner.is_overlapping( (1,2), (3,4) ))
        self.assertFalse(self.pruner.is_overlapping( (3,4), (1,2) ))
        self.assertTrue(self.pruner.is_overlapping( (1,3), (2,4) ))
        self.assertTrue(self.pruner.is_overlapping( (1,4), (2,3) ))
    
    def test_add_spane(self):
        self.pruner.add_spans( [1,2] )
        self.assertEqual(1, len(self.pruner.span_starts))
        self.assertEqual(1, len(self.pruner.span_ends))
        self.pruner._empty_spans()
        self.pruner.add_spans( [1,2], [100, 200] )
        self.assertEqual(2, len(self.pruner.span_starts))
        self.assertEqual(2, len(self.pruner.span_ends))
        self.pruner._empty_spans()
        self.pruner.add_spans( [100,200], [10, 20] )
        self.assertEqual(2, len(self.pruner.span_starts))
        self.assertEqual(2, len(self.pruner.span_ends))
        self.pruner._empty_spans()
        with self.assertRaises(ValueError):
            self.pruner.add_spans([1,4], [0,10])
        self.pruner._empty_spans()
        self.pruner.add_spans([1,4], [0,10])
        
        

